@@ -198,7 +198,7 @@ def match_api_version(
         if meta_loc != meta_srv:
             del server[meta_srv]
             to_download.append(meta_loc)
-            logit.warning(
+            logit.info(
                 f"Using non-default {meta.kind.upper()} endpoint "
                 f"<{meta_loc.apiVersion}>"
             )
@@ -571,9 +571,43 @@ def sort_plan(cfg: Config, plan: DeploymentPlan) -> Tuple[DeploymentPlan, bool]:
 
 
 def valid_label(label: str) -> bool:
-    """Return `True` if `label` is K8s and Square compatible."""
-    pat = re.compile(r"^[a-z0-9][-a-z0-9_.]*=[-A-Za-z0-9_.]*[A-Za-z0-9]$")
-    return pat.match(label) is not None
+    """Return `True` if `label` is K8s and Square compatible.
+
+    The `label` contains both key and value, eg `dh/app=morty`.
+
+    """
+    # K8s uses this regex to validate the three individual label components
+    # `name`, `part` and `suffix`.
+    # Example: "name=part/suffix" -> ("name", "part", "value").
+    pat = re.compile(r'^([A-Za-z0-9][-A-Za-z0-9_.]*)?[A-Za-z0-9]$')
+
+    try:
+        # Split `app=part/file` into (`app`, `part/value`).
+        assert label.count("=") == 1
+        tmp, value = label.split("=")
+
+        # Split on the "/"
+        # "part/value" -> ("part", "value")
+        # "value" -> ("", "value")
+        part, _, name = tmp.rpartition("/")
+        del label, _
+
+        # If the label contains a "/" then validate its "part" component.
+        assert len(name) > 0
+        if "/" in tmp:
+            assert pat.match(part)
+
+        # All components must be at most 64 characters long.
+        assert max(len(name), len(part), len(value)) < 64
+
+        # Validate the label and value.
+        assert pat.match(name)
+        assert pat.match(value)
+
+        # The label is valid.
+        return True
+    except AssertionError:
+        return False
 
 
 def apply_plan(cfg: Config, plan: DeploymentPlan) -> bool:
@@ -592,42 +626,47 @@ def apply_plan(cfg: Config, plan: DeploymentPlan) -> bool:
         logit.error(f"Invalid labels: {cfg.selectors.labels}")
         return True
 
-    try:
-        # Sort the plan according to `cfg.priority`.
-        plan, err = sort_plan(cfg, plan)
-        assert not err
+    # Sort the plan according to `cfg.priority`.
+    plan, plan_err = sort_plan(cfg, plan)
 
-        # Create properly configured Requests session to talk to K8s API.
-        k8sconfig, err = k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
-        assert not err
+    # Create properly configured "Requests" session to talk to the K8s API.
+    k8sconfig, k8s_err = k8s.cluster_config(cfg.kubeconfig, cfg.kubecontext)
 
-        # Convert "Selectors.kinds" to their canonical names.
-        cfg = translate_resource_kinds(cfg, k8sconfig)
-
-        # Create the missing resources.
-        for data_c in plan.create:
-            print(f"Creating {data_c.meta.kind.upper()} "
-                  f"{data_c.meta.namespace}/{data_c.meta.name}")
-            _, err = k8s.post(k8sconfig.client, data_c.url, data_c.manifest)
-            assert not err
-
-        # Patch the server resources.
-        patches = [(_.meta, _.patch) for _ in plan.patch if len(_.patch.ops) > 0]
-        for meta, patch in patches:
-            print(f"Patching {meta.kind.upper()} "
-                  f"{meta.namespace}/{meta.name}")
-            _, err = k8s.patch(k8sconfig.client, patch.url, patch.ops)
-            assert not err
-
-        # Delete the excess resources.
-        for data_d in plan.delete:
-            print(f"Deleting {data_d.meta.kind.upper()} "
-                  f"{data_d.meta.namespace}/{data_d.meta.name}")
-            _, err = k8s.delete(k8sconfig.client, data_d.url, data_d.manifest)
-            assert not err
-    except AssertionError:
+    # Abort if we could not get the plan or establish the K8s session.
+    if plan_err or k8s_err:
         return True
 
+    # Convert "Selectors.kinds" to their canonical names.
+    cfg = translate_resource_kinds(cfg, k8sconfig)
+
+    # Create the missing resources. Abort on first error.
+    for data_c in plan.create:
+        msg_res = f"{data_c.meta.kind.upper()} {data_c.meta.namespace}/{data_c.meta.name}"
+        print(f"Creating {msg_res}")
+        _, err = k8s.post(k8sconfig.client, data_c.url, data_c.manifest)
+        if err:
+            logit.error(f"Could not patch {msg_res}")
+            return True
+
+    # Patch the server resources. Abort on first error.
+    patches = [(_.meta, _.patch) for _ in plan.patch if len(_.patch.ops) > 0]
+    for meta, patch in patches:
+        msg_res = f"{meta.kind.upper()} {meta.namespace}/{meta.name}"
+        print(f"Patching {msg_res}")
+        _, err = k8s.patch(k8sconfig.client, patch.url, patch.ops)
+        if err:
+            logit.error(f"Could not patch {msg_res}")
+            return True
+
+    # Delete the excess resources. Abort on first error.
+    for data_d in plan.delete:
+        msg_res = f"{data_d.meta.kind.upper()} {data_d.meta.namespace}/{data_d.meta.name}"
+        print(f"Deleting {msg_res}")
+        _, err = k8s.delete(k8sconfig.client, data_d.url, data_d.manifest)
+        if err:
+            logit.error(f"Could not patch {msg_res}")
+            return True
+
     # All good.
     return False
 
